"use strict";

var _iniettore = _interopRequireWildcard(require("../../src/iniettore"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

describe('Given a context', () => {
  var VALUE_A = {
    value: 'a'
  };
  var parentProvideStub = jest.fn().mockReturnValue(42);
  var rootContext;
  describe('and a registered blueprint with some registered mappings', () => {
    var blueprintProviderStub = jest.fn().mockReturnValue(84);
    describe('when requesting a copy of the blueprint', () => {
      beforeAll(() => {
        function configureChildContext(map) {
          map('baz').to(blueprintProviderStub).as(_iniettore.TRANSIENT, _iniettore.SINGLETON, _iniettore.PROVIDER).injecting('bar');
        }

        rootContext = _iniettore.default.create(map => {
          map('bar').to(VALUE_A).as(_iniettore.VALUE);
          map('foo').to(configureChildContext).as(_iniettore.BLUEPRINT);
          map('pluto').to(parentProvideStub).as(_iniettore.TRANSIENT, _iniettore.SINGLETON, _iniettore.PROVIDER);
        });
      });
      it('should create a new child context', () => {
        var childContext = rootContext.get('foo');
        expect(childContext.get('bar')).toEqual(VALUE_A);
        expect(childContext.get('baz')).toEqual(84);
        expect(childContext.get('baz')).toEqual(84);
        expect(blueprintProviderStub).toHaveBeenCalledTimes(1);
        expect(blueprintProviderStub).toHaveBeenCalledWith(VALUE_A);
      });
      describe('multiple times', () => {
        it('should create a new child context each time', () => {
          var childContext1 = rootContext.get('foo');
          var childContext2 = rootContext.get('foo');
          expect(childContext1).not.toBe(childContext2);
        });
      });
    });
    describe('and a specified export alias', () => {
      beforeAll(() => {
        function configureChildContext(map) {
          map('baz').to(blueprintProviderStub).as(_iniettore.TRANSIENT, _iniettore.SINGLETON, _iniettore.PROVIDER).injecting('bar');
        }

        rootContext = _iniettore.default.create(map => {
          map('bar').to(VALUE_A).as(_iniettore.VALUE);
          map('foo').to(configureChildContext).as(_iniettore.BLUEPRINT).exports('baz');
          map('pluto').to(parentProvideStub).as(_iniettore.TRANSIENT, _iniettore.SINGLETON, _iniettore.PROVIDER);
        });
      });
      describe('when requesting a copy of the blueprint', () => {
        it('should return an instance of the export alias', () => {
          expect(rootContext.get('foo')).toEqual(84);
        });
      });
    });
  });
});