"use strict";

var _iniettore = _interopRequireWildcard(require("../../src/iniettore"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function noop() {}

describe('Given a child context', () => {
  var OBJECT = {};
  describe('when requesting an alias registered in the parent context', () => {
    it('should return the same binding as requested to the parent context', () => {
      var rootContext = _iniettore.default.create(map => {
        map('foo').to(OBJECT).as(_iniettore.VALUE);
      });

      var child = rootContext.createChild(noop);
      expect(child.get('foo')).toBe(rootContext.get('foo'));
    });
  });
  describe('when requesting the reference to the contexts', () => {
    it('should return the respective contexts', () => {
      var rootContext = _iniettore.default.create(noop);

      var child = rootContext.createChild(noop);
      expect(rootContext.get('$context')).toBe(rootContext);
      expect(child.get('$context')).toBe(child);
      expect(rootContext.get('$context')).not.toBe(child.get('$context'));
    });
  });
  describe('with a registered alias that shadows the one registered in the parent context', () => {
    describe('when requesting its alias from the child context', () => {
      var rootContext;
      beforeEach(() => {
        rootContext = _iniettore.default.create(map => {
          map('foo').to(OBJECT).as(_iniettore.VALUE);
        });
      });
      it('should retrieve the child context version', () => {
        var child = rootContext.createChild(map => {
          map('foo').to(42).as(_iniettore.VALUE);
        });
        expect(child.get('foo')).toEqual(42);
        expect(rootContext.get('foo')).toBe(OBJECT);
      });
    });
  });
  describe('when requesting an instance from a provider registered in the child context', () => {
    describe('with a dependency from a provider registered in the parent context', () => {
      var PARENT_INSTANCE = {};
      var CHILD_INSTANCE = {};
      var parentProviderStub = jest.fn().mockReturnValue(PARENT_INSTANCE);
      var chilProviderStub = jest.fn().mockReturnValue(CHILD_INSTANCE);
      var rootContext, childContext;
      beforeEach(() => {
        rootContext = _iniettore.default.create(map => {
          map('bar').to(parentProviderStub).as(_iniettore.TRANSIENT, _iniettore.SINGLETON, _iniettore.PROVIDER);
        });
        childContext = rootContext.createChild(map => {
          map('baz').to(chilProviderStub).as(_iniettore.TRANSIENT, _iniettore.SINGLETON, _iniettore.PROVIDER).injecting('bar');
        });
      });
      it('should create singleton instances in the respective contexts', () => {
        var baz = childContext.get('baz');
        expect(baz).toBe(CHILD_INSTANCE);
        expect(rootContext.get('bar')).toBe(PARENT_INSTANCE);
      });
    });
  });
});