"use strict";

var _iniettore = _interopRequireWildcard(require("../../src/iniettore"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

describe('Given iniettore', () => {
  describe('when creating a new context without a contribution function', () => {
    it('should throw an Error', () => {
      function testCase() {
        _iniettore.default.create();
      }

      expect(testCase).toThrow(/missing contribution function/i);
    });
  });
});
describe('Given a context', () => {
  var rootContext;
  describe('when registering a mapping with an invalid options combination', () => {
    it('should throw an Error', () => {
      function testCase() {
        _iniettore.default.create(map => {
          map('foo').to({}).as(_iniettore.SINGLETON, _iniettore.VALUE);
        });
      }

      expect(testCase).toThrow(/invalid flags combination/i);
    });
  });
  describe('when requesting an alias that has never registered before', () => {
    beforeAll(() => {
      rootContext = _iniettore.default.create(() => {});
    });
    it('should throw an Error', () => {
      function testCase() {
        rootContext.get('pluto');
      }

      expect(testCase).toThrow(new Error("'pluto' is not available. Has it ever been registered?."));
    });
  });
  describe('with a registered constructor', () => {
    class Foo {
      constructor() {
        throw new Error('Unexpected issue');
      }

    }

    beforeEach(() => {
      rootContext = _iniettore.default.create(map => {
        map('foo').to(Foo).as(_iniettore.CONSTRUCTOR);
      });
    });
    describe('when the constructor throw an Error', () => {
      it('should catch it and throw an Error specifing the failing component', () => {
        function testCase() {
          rootContext.get('foo');
        }

        expect(testCase).toThrow(new Error("Failed while resolving 'foo' due to:\n\tUnexpected issue"));
      });
    });
  });
  describe('with a registered singleton', () => {
    var DUMMY_INSTANCE = {
      dispose: () => {
        throw new Error('Unexpected issue');
      }
    };
    var providerStub = jest.fn().mockReturnValue(DUMMY_INSTANCE);
    beforeEach(() => {
      rootContext = _iniettore.default.create(map => {
        map('bar').to(providerStub).as(_iniettore.TRANSIENT, _iniettore.SINGLETON, _iniettore.PROVIDER);
      });
    });
    describe('when the releasing the singleton instance', () => {
      it('should enrich the message of any error the instance dispose method may throw', () => {
        rootContext.get('bar');

        function testCase() {
          rootContext.release('bar');
        }

        expect(testCase).toThrow(new Error("Failed while releasing 'bar' due to:\n\tUnexpected issue"));
      });
    });
    describe('when disposing the context itself', () => {
      it('should enrich the message of any error the instance dispose method may throw', () => {
        rootContext.get('bar');

        function testCase() {
          rootContext.dispose();
        }

        expect(testCase).toThrow(new Error("Failed while disposing 'bar' due to:\n\tUnexpected issue"));
      });
    });
  });
  describe('with a circular dependency', () => {
    describe('when requesting the corresponding alias', () => {
      it('should throw an Error', () => {
        class Bar {}

        rootContext = _iniettore.default.create(map => {
          map('bar').to(Bar).as(_iniettore.CONSTRUCTOR).injecting('bar');
        });

        function testCase() {
          rootContext.get('bar');
        }

        expect(testCase).toThrow(/Circular dependency/);
      });
    });
  });
});