"use strict";

var _iniettore = _interopRequireWildcard(require("../../src/iniettore"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

describe('Given a LAZY, SINGLETON, CONSTRUCTOR', () => {
  var constructorSpy = jest.fn();

  class Bar {
    constructor() {
      constructorSpy();
    }

  }

  describe('when registering it into the context', () => {
    it('should not create an instance', () => {
      _iniettore.default.create(map => {
        map('bar').to(Bar).as(_iniettore.LAZY, _iniettore.SINGLETON, _iniettore.CONSTRUCTOR);
      });

      expect(constructorSpy).not.toHaveBeenCalled();
    });
  });
  describe('when requesting the mapping for the first time', () => {
    var rootContext;
    beforeAll(() => {
      rootContext = _iniettore.default.create(map => {
        map('bar').to(Bar).as(_iniettore.LAZY, _iniettore.SINGLETON, _iniettore.CONSTRUCTOR);
      });
    });
    it('should create an instance', () => {
      rootContext.get('bar');
      expect(constructorSpy).toHaveBeenCalledTimes(1);
    });
  });
  describe('when requesting the mapping multiple times', () => {
    var rootContext;
    beforeAll(() => {
      rootContext = _iniettore.default.create(map => {
        map('bar').to(Bar).as(_iniettore.LAZY, _iniettore.SINGLETON, _iniettore.CONSTRUCTOR);
      });
    });
    it('should return the same instance', () => {
      var bar1 = rootContext.get('bar');
      var bar2 = rootContext.get('bar');
      expect(bar1).toBe(bar2);
    });
  });
  describe('when invoking rootContext.release with the corresponding mapping name', () => {
    var rootContext;
    beforeAll(() => {
      rootContext = _iniettore.default.create(map => {
        map('bar').to(Bar).as(_iniettore.LAZY, _iniettore.SINGLETON, _iniettore.CONSTRUCTOR);
      });
    });
    it('should not release the instance', () => {
      var bar1 = rootContext.get('bar');
      rootContext.release('bar');
      var bar2 = rootContext.get('bar');
      expect(bar1).toBe(bar2);
    });
  });
});
describe('Given a LAZY, SINGLETON, PROVIDER', () => {
  var barProvider = jest.fn();
  describe('when registering it into the context', () => {
    it('should not invoke the provider function', () => {
      _iniettore.default.create(map => {
        map('bar').to(barProvider).as(_iniettore.LAZY, _iniettore.SINGLETON, _iniettore.PROVIDER);
      });

      expect(barProvider).not.toHaveBeenCalled();
    });
  });
  describe('when requesting the mapping for the first time', () => {
    var rootContext;
    beforeAll(() => {
      rootContext = _iniettore.default.create(map => {
        map('bar').to(barProvider).as(_iniettore.LAZY, _iniettore.SINGLETON, _iniettore.PROVIDER);
      });
    });
    it('should invoke the provider function', () => {
      rootContext.get('bar');
      expect(barProvider).toHaveBeenCalledTimes(1);
    });
  });
  describe('when requesting the mapping multiple times', () => {
    var rootContext;

    function barProvider() {
      return {};
    }

    beforeAll(() => {
      rootContext = _iniettore.default.create(map => {
        map('bar').to(barProvider).as(_iniettore.LAZY, _iniettore.SINGLETON, _iniettore.PROVIDER);
      });
    });
    it('should return the instance created at the first request', () => {
      var bar1 = rootContext.get('bar');
      var bar2 = rootContext.get('bar');
      expect(bar1).toBe(bar2);
    });
  });
  describe('when invoking rootContext.release with the corresponding mapping name', () => {
    var rootContext;

    function barProvider() {
      return {};
    }

    beforeAll(() => {
      rootContext = _iniettore.default.create(map => {
        map('bar').to(barProvider).as(_iniettore.LAZY, _iniettore.SINGLETON, _iniettore.PROVIDER);
      });
    });
    it('should not release the instance', () => {
      var bar1 = rootContext.get('bar');
      rootContext.release('bar');
      var bar2 = rootContext.get('bar');
      expect(bar1).toBe(bar2);
    });
  });
});