'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createMapping;

var _utils = require("./utils");

var _createSimpleMappingFactory = _interopRequireDefault(require("./createSimpleMappingFactory"));

var _createSingletonMappingFactory = _interopRequireDefault(require("./createSingletonMappingFactory"));

var _createPropertyMapper = _interopRequireDefault(require("./createPropertyMapper"));

var _options = require("./options");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const factories = {
  [(0, _utils.generateMask)([_options.VALUE])]: (0, _createSimpleMappingFactory.default)(_utils.identity),
  [(0, _utils.generateMask)([_options.FUNCTION])]: (0, _createSimpleMappingFactory.default)(_utils.partial),
  [(0, _utils.generateMask)([_options.CONSTRUCTOR])]: (0, _createSimpleMappingFactory.default)(_utils.instanciate),
  [(0, _utils.generateMask)([_options.PROVIDER])]: (0, _createSimpleMappingFactory.default)(_utils.invoke),
  [(0, _utils.generateMask)([_options.TRANSIENT, _options.SINGLETON, _options.PROVIDER])]: (0, _createSingletonMappingFactory.default)(_utils.invoke, true),
  [(0, _utils.generateMask)([_options.TRANSIENT, _options.CONSTRUCTOR, _options.SINGLETON])]: (0, _createSingletonMappingFactory.default)(_utils.instanciate, true),
  [(0, _utils.generateMask)([_options.LAZY, _options.SINGLETON, _options.PROVIDER])]: (0, _createSingletonMappingFactory.default)(_utils.invoke),
  [(0, _utils.generateMask)([_options.LAZY, _options.SINGLETON, _options.CONSTRUCTOR])]: (0, _createSingletonMappingFactory.default)(_utils.instanciate),
  [(0, _utils.generateMask)([_options.EAGER, _options.SINGLETON, _options.PROVIDER])]: (0, _createSingletonMappingFactory.default)(_utils.invoke),
  [(0, _utils.generateMask)([_options.EAGER, _options.SINGLETON, _options.CONSTRUCTOR])]: (0, _createSingletonMappingFactory.default)(_utils.instanciate) // aliases

};
factories[(0, _utils.generateMask)([_options.INSTANCE])] = factories[(0, _utils.generateMask)([_options.VALUE])];

function createMapping(type, value, resolveDeps, releaseDeps) {
  if (!(type in factories)) {
    throw new Error('Invalid flags combination. See documentation for valid flags combinations.');
  }

  return (0, _createPropertyMapper.default)(factories[type], value, resolveDeps, releaseDeps);
}