"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.identity = identity;
exports.invoke = invoke;
exports.partial = partial;
exports.instanciate = instanciate;
exports.generateMask = generateMask;
exports.isEagerSingleton = isEagerSingleton;
exports.noop = noop;
Object.defineProperty(exports, "extractImplicitDependencies", {
  enumerable: true,
  get: function () {
    return _extractImplicitDependencies.default;
  }
});

var _extractImplicitDependencies = _interopRequireDefault(require("./extractImplicitDependencies"));

var _options = require("./options");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const EAGER_SINGLETON_PROVIDER = generateMask([_options.EAGER, _options.SINGLETON, _options.PROVIDER]);
const EAGER_SINGLETON_CONSTRUCTOR = generateMask([_options.EAGER, _options.SINGLETON, _options.CONSTRUCTOR]);

function identity(value) {
  return value;
}

function invoke(fn, deps) {
  return fn.apply(null, deps);
}

function partial(func, deps) {
  return function (...args) {
    return func.apply(this, deps.concat(args));
  };
}

function instanciate(Ctor, deps) {
  return new Ctor(...deps);
}

function generateMask(flags) {
  return flags.reduce((prev, curr) => prev | curr, 0);
}

function isEagerSingleton(type) {
  return [EAGER_SINGLETON_PROVIDER, EAGER_SINGLETON_CONSTRUCTOR].indexOf(type) > -1;
}

function noop() {}